// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IOFM__FORMATOB__HPP
#define BOOST__IOFM__FORMATOB__HPP
#  include <boost/io/format_objects.hpp>
#  include <boost/io/detail/config.hpp>
#  include <boost/noncopyable.hpp>

#  if !defined(BOOST_IOFM__NO_OUTPUT_DEDUCTION)
#     include <boost/io/deducer.hpp>
#  endif

#  include <iostream>

   namespace boost { namespace io
   {
      template< typename T, typename FormatType = char *, class Outputter = boost::io::basic_output >
      class formatob_t: public Outputter, public boost::noncopyable
      {
         public:
            const T &                  ob;
         public: // constructors
            inline           formatob_t( const formatob_t & fo ):
               Outputter( fo ),
               ob( fo.ob )
            {
            }
            inline           formatob_t( const T & val ) throw(): ob( val )
            {
            }
            inline           formatob_t( const T & val, const Outputter & o ) throw():
               Outputter( o ),
               ob( val )
            {
            }
      };

      template< typename CharT, class TraitsT, typename T, typename FormatType, typename Outputter >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT >         & os,
         const formatob_t< T, FormatType, Outputter > & fo
      )
      {
         return( fo( os, fo.ob ));
      }

      // format object generators

      template< typename T, class Outputter >
      inline formatob_t< T, typename Outputter::format_type, Outputter >
                                                 formatobout( const T & t, const Outputter & o )
      {
         return( formatob_t< T, typename Outputter::format_type, Outputter >( t, o ));
      }

#     if !defined(BOOST_IOFM__NO_OUTPUT_DEDUCTION) // automatic type deduction
         template< typename T >
         inline formatob_t< T, char *, typename deducer< T, char * >::type::outputter >
                                                 formatob( const T & t )
         {
            return( formatob_t< T, char *, typename deducer< T, char * >::type::outputter >
            (
               t,
               deducer< T, char * >::type::generate()
            ));
         }
#     else // no automatic deduction: use basic_output instead
         template< typename T >
         inline formatob_t< T >                  formatob( const T & t )
         {
            return( formatob_t< T >( t ));
         }
#     endif

      template< typename FormatType, typename T >
      inline formatob_t< T, FormatType >         formatobex( const T & t )
      {
         return( formatob_t< T, FormatType >( t ));
      }
   }}
#endif
