// (C) Copyright 2003: Reece H. Dunn

#include <iostream>
#include <boost/io/formatob.hpp>
#include <boost/io/stl_io.hpp>
#include <boost/numeric/interval.hpp>

#include "print_type.hpp"              // pretty-printing of data types

// test routine

template< typename Outputter, typename T >
void test_outputter( const Outputter & out, const T & value )
{
   std::cout << "results: ";
      out( std::cout, value );
   std::cout << '\n';
}

// main test program

int main()
{
   int                                 i  = 5;
   std::complex< float >               cf = std::complex< float >( 1.5f, 2.2f );
   int                                 ia[ 4 ] = { 1, 2, 3, 4 };

   std::list< char >                   lc;
   lc.push_back( 'A' );
   lc.push_back( 'B' );
   lc.push_back( 'C' );

   std::cout << "testing: <boost/io/format_output/basic_output.hpp>" << '\n' << '\n';

   boost::io::basic_output             out1;

   test_outputter( out1, i );
   test_outputter( out1, cf );

   std::cout << '\n'; // test generator:

   test_outputter( boost::io::basicfmt(), i );

   std::cout << '\n' << "testing: <boost/io/format_output/wrapped_output.hpp>" << '\n' << '\n';

   boost::io::wrapped_output< char * > out2;
   out2.format( "<item>", "</item>" );

   test_outputter( out2, i );
   test_outputter( out2, cf );

   std::cout << '\n';

   out2.format( "==> ", "" );

   test_outputter( out2, i );
   test_outputter( out2, cf );

   std::cout << '\n'; // test generator:

   test_outputter( boost::io::wrappedfmt().format( "<", ">" ), i );

   std::cout << '\n' << "testing: <boost/io/format_output/pair_output.hpp>" << '\n' << '\n';

   boost::io::pair_output< char * >    out3;
   out3.format( " : " );

   test_outputter( out3, cf );

   std::cout << '\n'; // test generator:

   test_outputter( boost::io::pairfmt(), cf );

   std::cout << '\n' << "testing: <boost/io/format_output/static_nary_output.hpp>" << '\n' << '\n';

#  if !defined(BOOST_IOFM__NO_LIB_QUATERNION)
      boost::math::quaternion< double > q( 1.1, 2.2, 3.3, 4.4 );

      boost::io::static_nary_output< char * > out4;

      test_outputter( out4, q );

      std::cout << '\n'; // test generator:

      test_outputter( boost::io::naryfmt().format( " :: " ), q );
#  endif

   std::cout << '\n' << "testing: <boost/io/format_output/array_output.hpp>" << '\n' << '\n';

   boost::io::array_output< char * >   out5;

   test_outputter( out5( 4 ),    ia );
   test_outputter( out5( 1, 2 ), ia );

   std::cout << '\n'; // test generator:

   test_outputter( boost::io::arrayfmt( 4    ).format( "{ ", " };" ),     ia );
   test_outputter( boost::io::arrayfmt( 1, 2 ).format( "< ", " >", "|" ), ia );

   std::cout << '\n' << "testing: <boost/io/format_output/container_output.hpp>" << '\n' << '\n';

   boost::io::container_output< char * > out6;

   test_outputter( out6, lc );

   std::cout << '\n'; // test generator:

   test_outputter( boost::io::containerfmt().format( " | " ), lc );

   std::cout << '\n' << "testing: <boost/io/formatob.hpp>" << '\n' << '\n';

   std::cout << "formatob: " << boost::io::formatob( i  ) << '\n';
   std::cout << "formatob: " << boost::io::formatob( cf ) << '\n';
   std::cout << "formatob: " << boost::io::formatobout( cf, boost::io::pairfmt()) << '\n';
#  if !defined(BOOST_IOFM__NO_LIB_QUATERNION)
      std::cout << "formatob: " << boost::io::formatobout( q, boost::io::naryfmt()) << '\n';
#  endif
   std::cout << "formatob: " << boost::io::formatobout( ia, boost::io::arrayfmt( 4 )) << '\n';
   std::cout << "formatob: " << boost::io::formatobout( lc, boost::io::containerfmt()) << '\n';
   
#  if !defined(BOOST_IOFM__NO_OUTPUT_DEDUCTION)
      std::cout << '\n' << "automatic outputter deduction:" << '\n' << '\n';

      std::cout << "formatob: " << boost::io::formatob( i  ) << '\n';
      std::cout << "formatob: " << boost::io::formatob( cf ) << '\n';
#     if !defined(BOOST_IOFM__NO_LIB_QUATERNION)
         std::cout << "formatob: " << boost::io::formatob( q ) << '\n';
#     endif
      std::cout << "formatob: " << boost::io::formatob( ia ) << '\n';
      std::cout << "formatob: " << boost::io::formatob( lc ) << '\n';
#  endif

   std::cout << '\n' << "testing: <boost/io/stl_io.hpp>" << '\n' << '\n';

   std::cout << "list = " << lc << '\n';

   return( 0 );
}
