// (C) Copyright 2003: Reece H. Dunn

#include <iostream>
#include <boost/io/formatter.hpp>

#include "print_type.hpp"              // pretty-printing of data types

// test routine

template< typename Formatter >
void test_openclose_formatter( const Formatter & fmt )
{
   std::cout << "formatter type: boost::io::openclose_formatter< " << boost::io::print_type< typename Formatter::format_type >() << " >" << '\n'
             << "   open  = '" << fmt.open()  << "'" << '\n'
             << "   close = '" << fmt.close() << "'" << '\n' << '\n';
}

template< typename Formatter >
void test_formatter( const Formatter & fmt )
{
   std::cout << "formatter type: boost::io::formatter< " << boost::io::print_type< typename Formatter::format_type >() << " >" << '\n'
             << "   open       = '" << fmt.open()      << "'" << '\n'
             << "   close      = '" << fmt.close()     << "'" << '\n'
             << "   separator  = '" << fmt.separator() << "'" << '\n' << '\n';
}

// main test program

int main()
{
   std::cout << "testing: <boost/io/detail/openclose_formatter.hpp>" << '\n' << '\n';

   boost::io::openclose_formatter< char >
                                       c_ocfmt;
   boost::io::openclose_formatter< char * >
                                       ca_ocfmt;
   boost::io::openclose_formatter< std::string >
                                       str_ocfmt;

   test_openclose_formatter( c_ocfmt   );
   test_openclose_formatter( ca_ocfmt  );
   test_openclose_formatter( str_ocfmt );

   test_openclose_formatter( c_ocfmt.format(   '<',  '>'  ));
   test_openclose_formatter( ca_ocfmt.format(  "< ", " >" ));
   test_openclose_formatter( str_ocfmt.format( "< ", " >" ));

   std::cout << "testing: <boost/io/detail/formatter.hpp>" << '\n' << '\n';

   boost::io::formatter< char >        c_fmt;
   boost::io::formatter< char * >      ca_fmt;
   boost::io::formatter< std::string > str_fmt;

   test_formatter( c_fmt   );
   test_formatter( ca_fmt  );
   test_formatter( str_fmt );

   test_formatter( c_fmt.format( '<', '>' ));
   test_formatter( c_fmt.format( '{', '}', '|' ));
   test_formatter( c_fmt.format( ':' ));

   test_formatter( ca_fmt.format(  "< ", " >" ));
   test_formatter( ca_fmt.format(  "{ ", " }", "|" ));
   test_formatter( ca_fmt.format(  " : " ));

   test_formatter( str_fmt.format( "< ", " >" ));
   test_formatter( str_fmt.format(  "{ ", " }", "|" ));
   test_formatter( str_fmt.format(  " : " ));

   return( 0 );
}
